

/**
 * \file
 *         Testing writing to flash
 * \author
 *         Moritz Strbe <morty@cs.fau.de>
 */

#include "contiki.h"

#include <stdio.h>
#include <stdint.h>
#include <dev/flash.h>
#include <dev/uart1.h>
#include "misc_align.h"


extern char __data_end_rom[];
extern char __vectors_start[];

static char *flash_start;
static char *flash_end;
#define flashlen ((size_t) (flash_end - flash_start))


static void init(void){
  flash_start = (char *)(ALIGN_ROM_NEXT((uintptr_t)__data_end_rom));
  flash_end = (char*)ALIGN_ROM_PREV((uintptr_t)__vectors_start);
}

static size_t
fill_flash(void *start,  size_t len)
{
  unsigned short *lcldest = start;
  size_t written = 0;
  unsigned short ow;
  uint8_t *owptr = (uint8_t*)(&ow);


  flash_setup();

  while( (len & ~0x1) > written) {
    owptr[0] = (uint8_t)written;
    owptr[1] = (uint8_t)(written+1);
    flash_write(lcldest++, ow);
    written += 2;
  }
  flash_done();
  IFG1 |= UTXIFG0;
  return written;
}

static uint16_t
checkflash(void *start,  uint16_t len, uint16_t limit)
{
  uint16_t ctr;
  uint16_t rv = 0;
  uint16_t octr = 0;
  uint8_t *cptr = start;
  for(ctr = 0; ctr < len; ctr++){
    if(*cptr != (uint8_t)ctr){
      rv = 1;
      printf("Unexpected value at %.4x: Read %.2x instead of %.2x\n", (uint16_t)cptr, (uint8_t)*cptr, (uint8_t)ctr);
      octr ++;
      if(octr == limit){
        printf("To many errors");
        return rv;
      }

    }
    cptr++;
  }
  return rv;
}


static void erasearea_flash(void *start, size_t size)
{

  flash_setup();
  while (size >= ROM_ERASE_UNIT_SIZE) {
    flash_clear(start);
    size -= ROM_ERASE_UNIT_SIZE;
    start = (char*)start + ROM_ERASE_UNIT_SIZE;
  }
  flash_done();
}




static void
flash_contents(void *start, size_t size, uint16_t outp)
{
  uint8_t * cptr;
  uint16_t ctr;
  uint8_t lastc;
  uint16_t octr = 0;
  ctr = 0;
  cptr = start;
  lastc = *cptr;

  printf("Scanning flash from %.4x to %.4x\n", (uint16_t)cptr, (uint16_t)start + size);
  //flush();


  for (; (uint16_t)cptr < (uint16_t)start + size; cptr++) {
    if(lastc == *cptr) {
      putchar('.');
      ctr++;
      continue;
    }
    printf("\n%i Chars of %.2x\n", ctr, lastc);
    lastc = *cptr;
    ctr = 1;
    octr ++;
    if(octr == outp) return;

  }
  if(ctr) printf("\n%i Chars of %.2x\n", ctr, lastc);
  printf("Done\n");
}

/*---------------------------------------------------------------------------*/
PROCESS(flash_test_process, "Flash-Test process");
AUTOSTART_PROCESSES(&flash_test_process);
/*---------------------------------------------------------------------------*/
PROCESS_THREAD(flash_test_process, ev, data)
{
  static uint16_t ctr;

  PROCESS_BEGIN();
  init();
  printf("Starting flash test\n\n");
  printf("First address to write: %.4x\n", (uint16_t)flash_start);
  printf("Last address to write: %.4x\n\n", (uint16_t)flash_end);
  flash_contents(flash_start, flashlen, 10);
  printf("Erasing flash\n");
  erasearea_flash(flash_start, flashlen);

  flash_contents(flash_start, flashlen, 10);
  printf("Writing data\n");;
  fill_flash(flash_start, flashlen);

  printf("Testing Data\n");
  if(checkflash(flash_start, flashlen, 10)){
    printf("Test Failed\n");
  }

  printf("Wait some time\n");
  for(ctr = 10000; ctr > 0; ctr--){
    if(!(ctr % 10)) printf("%i\n", ctr);
    PROCESS_PAUSE();
  }

  printf("Testing Data\n");
  if(checkflash(flash_start, flashlen, 10)){
    printf("Test Failed\n");
  } else {
    printf("Test OK\n");
  }
  
  PROCESS_END();
}
/*---------------------------------------------------------------------------*/
